/*global define*/
/*jslint plusplus:true*/
define("ckeditor/plugins/eplineheight/plugin", [
    "ckeditor/ckeditor",
    "$dict!ep/dict"
], function (CKEDITOR, epDict) {
    /*
     * @dictionary      ep.dict
     *
     * @translation     {LineHeight}
     */
    'use strict';

    var lineHeights = ['75', '100', '125', '150', '200', '300'],

        lineHeightsLength = lineHeights.length,

        tLineHeight = epDict.translate('LineHeight');

    function getLineHeight(editor, selection) {
        var value,
            ranges = selection.getRanges(),
            enterMode = editor.config.enterMode,
            iterator,
            block,
            i,
            j,
            len;

        if (editor.document.getData().length === 0) {
            return;
        }

        for (i = 0, len = ranges.length; i < len; i++) {
            iterator = ranges[i].createIterator();
            iterator.enlargeBr = enterMode !== CKEDITOR.ENTER_BR;

            while ((block = iterator.getNextParagraph(enterMode === CKEDITOR.ENTER_P ? 'p' : 'div'))) {
                for (j = 0; j < lineHeightsLength; j++) {

                    if (block.hasClass('line-height-' + lineHeights[j])) {
                        value = lineHeights[j];
                        break;
                    }

                }
            }
        }

        return value;
    }

    function lineHeightCommand(value) {
        this.value = value;
    }

    lineHeightCommand.prototype = {
        exec : function(editor) {
            var selection = editor.getSelection(),
                ranges = selection.getRanges(),
                enterMode = editor.config.enterMode,
                iterator,
                block,
                i,
                len,
                value;

            if (!selection) {
                return;
            }

            for (i = 0, len = ranges.length; i < len; i++) {
                iterator = ranges[i].createIterator();
                iterator.enlargeBr = enterMode !== CKEDITOR.ENTER_BR;

                while ((block = iterator.getNextParagraph(enterMode === CKEDITOR.ENTER_P ? 'p' : 'div'))) {
                    value = getLineHeight(editor, selection);

                    if (value) {
                        block.removeClass('line-height-' + value);
                        editor.getCommand(editor.getMenuItem(value).command).setState(CKEDITOR.TRISTATE_OFF);
                    }

                    block.addClass('line-height-' + this.value);
                    this.setState(CKEDITOR.TRISTATE_ON);
                }
            }
        }
    };

    CKEDITOR.plugins.add('eplineheight', {
        requires : ['menubutton'],

        init: function (editor) {
            var menuGroup = 'lineHeight',
                uiMenuItems = {},
                lineHeight,
                commandName,
                i;

            // add menu group
            editor.addMenuGroup(menuGroup);

            // create menu entries, commands and event listener for different line heights
            for (i = 0; i < lineHeightsLength; i++) {
                lineHeight = lineHeights[i];
                commandName = 'lineHeight' + [lineHeights[i]];

                editor.addCommand(commandName, new lineHeightCommand(lineHeight));

                uiMenuItems[lineHeights[i]] = {
                    label : lineHeight + '%',
                    command : commandName,
                    group : menuGroup
                };
            }

            // add items to the menu
            editor.addMenuItems(uiMenuItems);

            // create menu button
            editor.ui.add('eplineheight', CKEDITOR.UI_MENUBUTTON, {
                label : tLineHeight,
                title : tLineHeight,
                modes : { wysiwyg : 1 },
                icon: this.path + 'images/lineheight.png',
                onRender: function () {
                    editor.on('selectionChange', function(ev) {
                        var hasLineHeight = getLineHeight(editor, ev.data.selection);

                        // set highlight state if line-height is set for current selection
                        this.setState(hasLineHeight ? CKEDITOR.TRISTATE_ON : CKEDITOR.TRISTATE_OFF);
                    }, this);
                },
                onMenu : function () {
                    return {
                        75  : CKEDITOR.TRISTATE_OFF,
                        100 : CKEDITOR.TRISTATE_OFF,
                        125 : CKEDITOR.TRISTATE_OFF,
                        150 : CKEDITOR.TRISTATE_OFF,
                        200 : CKEDITOR.TRISTATE_OFF,
                        300 : CKEDITOR.TRISTATE_OFF
                    };
                }
            });
        }
    });

    return CKEDITOR;

});