#========================================================================================
# §package      DE_EPAGES::RemoteSearch::t::SuggestClient
# §state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::SuggestClient;
use base Test::Class;

use strict;
use warnings;

use DE_EPAGES::RemoteSearch::API::Object::SuggestClient;
use DE_EPAGES::EPagesJ::API::Object::HTTPClient;
use HTTP::Headers;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::More;
use Test::MockObject;
use utf8;
use JSON;

my $HTTP_Response_Content = '';

sub testSimpleSuggest : Test(no_plan) {
    my $self   = shift;
    my $Local = << 'EOL';
{
  "products":[{"productId":1234,"name":"MyProduct","imagePath":"http://localhost:8088/static/image.jpg"}],
  "manufacturers":[{"matches":1,"name":"Manufacturer","searchLink":null}],
  "categories":[{"matches":1,"name":"Cat 1","searchLink":null}],
  "link":{"relationship":"self",
  "href":"","type":"application/json","hrefURI":null},
  "empty":false
}
EOL

    $HTTP_Response_Content = $Local;
    my $Client = DE_EPAGES::RemoteSearch::API::Object::SuggestClient->new(
        'BaseURL'    => 'http://localhost:8088/rs-test/',
        'Datasource' => 'Store',
        'Resource'   => 'GUID',
        'HttpClient' => $self->_mockHTTPClient( { 'get' => \&_assertSimpleSuggestContents } )
    );
    my $Response = $Client->suggest('myquery', 'de');
    isa_ok($Response, 'HTTP::Response');
    is($Response->decoded_content, $Local, 'content ok');
}

sub testSuggestAsHash : Test(no_plan) {
    my $self   = shift;
    $HTTP_Response_Content =<< 'EOL';
{
  "products":[{"productId":1234,"name":"MyProduct",
  "imagePath":"http://localhost:8088/static/image.jpg"}],
  "manufacturers":[{"matches":1,"name":"Manufacturer","searchLink":null}],
  "categories":[{"matches":1,"name":"Cat 1","searchLink":null}],
  "link":{"relationship":"self",
  "href":"","type":"application/json","hrefURI":null},
  "empty":false
}
EOL
    my $Client = DE_EPAGES::RemoteSearch::API::Object::SuggestClient->new(
        'BaseURL'    => 'http://localhost:8088/rs-test/',
        'Datasource' => 'Store',
        'Resource'   => 'GUID',
        'HttpClient' => $self->_mockHTTPClient( { 'get' => \&_assertSimpleSuggestContents } )
    );
    my $Response = $Client->suggestAsHash('myquery', 'de');
    is($Response->{products}->[0]->{productId}, 1234, 'check hash contents.'); ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
}

sub testSuggestAsHashWithUmlauts : Test(no_plan) {
    my $self   = shift;
    $HTTP_Response_Content =<< 'EOL';
{
  "products":[{"productId":1234,"name":"Schmüsöbäß",
  "imagePath":"http://localhost:8088/static/image.jpg"}],
  "manufacturers":[],
  "categories":[],
  "link":{"relationship":"self",
  "href":"","type":"application/json","hrefURI":null},
  "empty":false
}
EOL
    my $Client = DE_EPAGES::RemoteSearch::API::Object::SuggestClient->new(
        'BaseURL'    => 'http://localhost:8088/rs-test/',
        'Datasource' => 'Store',
        'Resource'   => 'GUID',
        'HttpClient' => $self->_mockHTTPClient( { 'get' => \&_assertSimpleSuggestContents } )
    );
    my $Response = $Client->suggestAsHash('myquery', 'de');
    is($Response->{products}->[0]->{name}, 'Schmüsöbäß', 'check umlauts.');
}

sub _mockHTTPClient {
    my $self           = shift;
    my ($hMockMethods) = @_;
    my $mockRemote     = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

# Fake method for HTTPClient->get($URL, $hHeader)
# Method need extra data, which is supplied via global $HTTP_Response_Content   
sub _assertSimpleSuggestContents {
    shift;
    my ( $URL, $hHeader ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID/suggest?q=myquery&lang=de', 'get path for SimpleSuggest' );
    # get $Content provided by test method
    my $Content = $HTTP_Response_Content;
    utf8::encode($Content);
    # delete global data; ensure no reuse (guard againts coding errors; no implicit dependencies etc.) 
    $HTTP_Response_Content = '';
    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), HTTP::Headers->new('Content-Type' => 'application/json; charset=utf-8'), $Content );
}

####### Main - not part of Test::Class ############
package main;    ## no critic (ProhibitMultiplePackages)

use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub { Test::Class->runtests;}
);
